//
// here are a few miscellaneous functions
//

#include "gogo.h"

// TEXT() may be used to write characters on the graphics
// window.  the size of the chars is specified by the value of
// "font", which must be an integer 0 <= font <= 3.  s is some
// string like "hi there", which may include newline chars
// (\n), which will cause a line feed and carriage return,
// moving to a new position below the already output text line
// by an amount determined by the value of "font".  a floating
// point number x may be added if desired.
//
// typical usages now include these:
//
// TEXT(0, "This will be output as\ntwo lines of text.");
// TEXT(1, "This will be in slightly samller char size.");
// TEXT(3, "This is the smallest character size.");
// TEXT(0, "The value of x = ", x);
//
// additional choices, involving various combinations of
// strings and floats may be added in the future.
//

void TEXT(uns font, string s)
{
    GMODE(off);           // enter text mode in graphics window
    uns bl_skip = 22;     // baseline skip
    switch (font){
        case 0:
            out(ofs, "\0338"); // large char size
            bl_skip = 22;
            break;
        case 1:
            out(ofs, "\0339"); // medium char size
            bl_skip = 21;
            break;
        case 2:
            out(ofs, "\033:"); // small char size
            bl_skip = 15;
            break;
        case 3:
            out(ofs, "\033;"); // tiny char size
            bl_skip = 13;
            break;
        default:
            err(font, ": unallowed font for TEXT function");
    }
    vector<string> vec_str = form_string(s); // transform any \n chars
    for (uns i=0 ; i < vec_str.size() ; i++){
        out(ofs, vec_str[i]); // output a line of text
        newline(bl_skip);     // add an ersatz newline
    }
    out(ofs, "\0338 \010");
        /* the above line is inserted so as to produce
        a "0 SetCharStyle" command in the file produced by the
        ps4014 translator, by (a) returning to the large char
        size, (b) printing a space, and (c) printing a backspace
        char.  this will be useful in producing rotated and
        greek text chars in the postscript translations. */
    GMODE(on);                 // turn GMODE back on
}

void TEXT(uns font, string s, flt x)
{
    GMODE(off);           // enter text mode in graphics window
    uns bl_skip = 22;     // baseline skip
    switch (font){
        case 0:
            out(ofs, "\0338"); // large char size
            bl_skip = 22;
            break;
        case 1:
            out(ofs, "\0339"); // medium char size
            bl_skip = 21;
            break;
        case 2:
            out(ofs, "\033:"); // small char size
            bl_skip = 15;
            break;
        case 3:
            out(ofs, "\033;"); // tiny char size
            bl_skip = 13;
            break;
        default:
            err(font, ": unallowed font for TEXT function");
    }
    vector<string> vec_str = form_string(s); // transform any \n chars
    for (uns i=0 ; i < vec_str.size() ; i++){
        newline(bl_skip);     // add an ersatz newline
        out(ofs, vec_str[i]); // output a line of text
    }
    out(ofs, x);
    out(ofs, "\0338 \010");
        /* the above line is inserted so as to produce
        a "0 SetCharStyle" command in the file produced by the
        ps4014 translator, by (a) returning to the large char
        size, (b) printing a space, and (c) printing a backspace
        char.  this will be useful in producing rotated and
        greek text chars in the postscript translations. */
    GMODE(on);                 // turn GMODE back on
}
vector<string> form_string(const string ss)
{
    vector<string> vec_str;  // this will be returned
    string s = ss;  // copy to manipulate
    string::size_type k = 0;
    for (uns i=0 ; (k = s.find("\n")) < string::npos ; i++){
        vec_str.push_back(s.substr(0, k));
        s.erase(0, k+1);
    }
    vec_str.push_back(s.substr(0, k)); // store the last component
    return vec_str;
}

void newline(uns bl_skip)
{
    vector<int> g (2); // graphic screen coords of point
    GMODE(on);
    g[0] = glast[0];
    g[1] = glast[1] - bl_skip;
    MG(g);
    GMODE(off);
}
// ================================================================
//
// this enters the "GIN" (graphics input) mode, turning on the
// cross-hair cursor in the graphics window.  the cross-hair
// intersection may then be moved using the mouse to some desired
// point on the screen.  then, when any key or a mouse button is
// pressed (not ENTER or SPACE), the graph coordinates of that
// point are read and stored in the 2-component vector pointed
// to by g, which is then returned.  the "GIN" terminator should be
// set to "CR" in the setup of the GraphOn terminal.  For an X
// terminal, put the line "Xterm.Tek4014.ginTerminator: CRonly"
// in the .Xresources file, so the action will be immediate.
// See the xterm man page for additional limited information.
// If the input is not read correctly the function asks the user to try
// again; after several tries, it exits with an error message.  as will
// be seen with the code below, the GIN mode does not always work as
// advertised, but this will correct for the common error that only four
// chars (not five) are read when a key is pressed.  pressing the
// ENTER key will put the read char string on the screen; pressing the
// SPACE key will generally work, however.  pressing number pad keys
// will generally not work.  this behavior may work differently for
// different versions of xterm.
//

vector<int> GIN()       // returns 2-component int g-vector
{
    cchr MASK=0xdf;     // 1101 1111 turns off bit 6
    char xhi=0, xlo=0, yhi=0, ylo=0;
    string s = "";
    vector<int> g (2);  // graph coordinate vector
    cout << "\033\032";  // enter GIN mode: esc ctrl-z
    cin >> s;            // get response to key press in s
    if (s.size() == 5){  // s[0] should be key's ASCII code
        xhi = s[1];      // (normal behavior)
        xlo = s[2];
        yhi = s[3];
        ylo = s[4];
    }
    else if (s.size() == 4){  // this happens sometimes, because
        s.insert(0,"i"); // it fails to get key pressed
        xhi = s[1];
        xlo = s[2];
        yhi = s[3];
        ylo = s[4];
    }
    else {
        AWIN();
        cout << "Graphic input failed, point may be outside window,\n"
                "so I'm returning to your previous point.\n";
        GWIN();
        g = glast;
        return g;
    }
    xlo &= MASK;
    g[0] = (xhi & MASK) << 5 | xlo;  // xg
    ylo &= MASK ;
    g[1] = (yhi & MASK) << 5 | ylo;  // yg
    if (g[0] < w[0] || g[0] > w[2] || // check g for validity
        g[1] < w[1] || g[1] > w[3]){
        AWIN();
        cout <<
            "  I got a point that seems outside the current\n"
            "  window, so I'm using your previous point.\n"
            "  You might want to try again.\n";
        GWIN();
        g = glast;
    }
//  AWIN();
//  cout << "g[0] = " << g[0] << "\t";
//  cout << "g[1] = " << g[1] << "\n";
//  GWIN();
    return g;
}
// ================================================================
//
// when used in the graphics window, gets the status byte and
// the position of the lower left corner of the cursor.  the
// coordinates of this position are assigned to the 2-component
// vector pointed to by g.  unlike the GIN() function, does not
// enter the GIN mode.  note that memory must be allocated for
// *g in the calling function.  Be sure to put the line
// "Xterm.Tek4014.ginTerminator: CRonly" in your .Xresources file.
//

vector<int> GETXY()
{
    cchr MASK=0xdf; // 1101 1111 turns off bit 6
    char xhi=0, xlo=0, yhi=0, ylo=0;
    string s = "";  // empty string
    cout << "\033\005";  // request status report (esc ctrl-E)
    cin >> s;            // get the response
                         // s[0] is status byte, (4)
    if (s.size() == 5){  // (normal behavior)
        xhi = s[1];
        xlo = s[2];
        yhi = s[3];
        ylo = s[4];
    }
    else if (s.size() == 4){  // may happen sometimes
                              // if status byte is lost
        s.insert(0,"4");
        xhi = s[1];
        xlo = s[2];
        yhi = s[3];
        ylo = s[4];
    }
    else {               // wrong s.size(), so quit
        err("GETXY failed.  Normally string length will be 5\n"
            "   (or sometimes 4); but this got a string length\n"
            "   of ", s.length());
    }
    vector<int> g (2);   // graph coordinate vector
    xlo &= MASK;
    g[0] = (xhi & MASK) << 5 | xlo;    // xg
    ylo &= MASK ;
    g[1] = (yhi & MASK) << 5 | ylo;    // yg
//    AWIN();
//    cout << "s.length() = " << s.length() << '\n';
//    cout << "s = " << s << '\n';
//    cout << "g[0] = " << g[0] << "\t";
//    cout << "g[1] = " << g[1] << "\n";
//    GWIN();
    return g;
}
// ================================================================
//
// u2g converts user unit float vector u to
// tek graph unit int vector g using the
// transformation matrix trm created by SCALE().
// note the use of 0.5 for proper rounding.
//

vector<int> u2g(vector<flt>& u)
{
//    fmatrix t = trm;  // for brevity; trm is global
    vector<int> g (2);
    g[0] = int(trm[0][0]*u[0] + trm[0][2] + 0.5);
    g[1] = int(trm[1][1]*u[1] + trm[1][2] + 0.5);
    return g;
}
// ================================================================
//
// g2u converts tek graph int vector g to
// user float vector u, returning the latter.
// transformation matrix trm created by SCALE().
//

vector<flt> g2u(vector<int>& g)
{
//    fmatrix t = trm;  // for brevity; trm is global
    vector<flt> u (3);
    u[0] = float(g[0]) - trm[0][2];
    u[0] /= trm[0][0];
    u[1] = float(g[1]) - trm[1][2];
    u[1] /= trm[1][1];
    u[2] = 1.0;       // for shifting origin
    return u;
}
// ================================================================
//
// calculates length of window diagonal
// in graph units converted to float
//

flt windiag(void)
{
    flt wx, wy, z;

    wx = (float)(w[2] - w[0]);   // win width in graph units
    wy = (float)(w[3] - w[1]);   // win height in graph units
    z  = sqrt(wx*wx + wy*wy);    // z is length of diagonal
    return z ;
}
// ================================================================
