//
// this group of functions: (INITGRAPH(), CLEAR(),
// WINDOW(), SCALE(), BORDER(), STYLE() and END()
// are the ones generally used for setting up the
// graphics environment for graph display in the tek
// window, except that END() is used to terminate
// the graphics display.
//

#include "gogo.h"

//
// the usual command to start a graphics routine.
// it sets the xterm flag to TRUE if terminal is xterm,
// sets boundaries for a default window, and switches to
// the graphics window in the graphics mode.
// 

void INITGRAPH()
{
    static int init = false;

    extern vector<int> glast; // 2d vector grid coords
    if (init) glast.clear();  // free any previous memory
    init = true;              // allocation indicator

    // check to make sure we're using an xterm,
    // and if we are, set xterm flag true:
    // only xterm sets XTERM_VERSION
    char* xterm_version = getenv("XTERM_VERSION");
    if (xterm_version) {
        cerr << "xterm_version = " << xterm_version << endl;
        cerr << "hit \'x\' or \'q\' to exit\n";
        xterm = true;
    }
    else {
        err("not an xterm...\n"
        "   you need to run lorenz from an xterm");
    }
    GWIN();                        // switch to graph window
    GMODE(on) ;                    // put in graphics mode
    WINDOW(0.0, 0.0, 10.23, 7.81); // default largest window
    SCALE(0.0, 0.0, 10.0, 10.0);   // default scaling
    STYLE(0);                      // ensure solid line default
}
// ==========================================================

//
// clears graphics screen
//

void CLEAR(void)
{
    if (oflg == term){            // don't clear if file output
        if (gmode){
//            GMODE(off);
            out(ofs, "\033\014"); // works only if GMODE(off)
//            GMODE(on);
        }
        else {
            out(ofs, "\033\014"); // leave GMODE(on)
        }
    }
}
// ==========================================================

//
// determines the boundary of the area to be scaled,
// that is, the window.  the arguments of this
// function, specifying the coordinates of the lower
// left and upper right corners of the desired window,
// are in units of inches, the size of the resulting
// window appearing on PostScript output (not inches
// on the monitor screen).
//
void WINDOW(flt llxi, flt llyi, flt urxi, flt uryi)
{
    w[0] = int(PTS_PER_IN * llxi + 0.5); // w is global
    w[1] = int(PTS_PER_IN * llyi + 0.5);
    w[2] = int(PTS_PER_IN * urxi + 0.5);
    w[3] = int(PTS_PER_IN * uryi + 0.5);
    if (w[0] < 0 || w[0] > URX_MAX ||
        w[1] < 0 || w[1] > URY_MAX ||
        w[2] < 0 || w[2] > URX_MAX ||
        w[3] < 0 || w[3] > URY_MAX ){
        AWIN();            // switch to alphanumeric window
        err("The window boundaries are out of range.\n"
            "You must have\n\n"
            "  0.0 <= llx < urx <= 10.23\n"
            "  0.0 <= lly < ury <=  7.81\n\n");
    }
}
// ==========================================================
//
// creates a 2 by 3 transformation matrix trm that
// scales and shifts the user coordinates of the user
// window (llx, lly, urx, ury), so as to map onto the
// window graph coordinates specified by the WINDOW
// function.
//
// trm is global, defined in defs.c
// w is global, defined in defs.c
//

void SCALE(flt llx, flt lly, flt urx, flt ury)
{
    trm[0][0] = (w[2] - w[0])/(urx - llx); // xscale
    trm[1][1] = (w[3] - w[1])/(ury - lly); // yscale
    trm[0][2] = w[0] - llx * trm[0][0];    // xorg
    trm[1][2] = w[1] - lly * trm[1][1];    // yorg
}
// ==========================================================
//
// draws a border at the window boundaries
//

void BORDER(void)
{
    MG(w[0], w[1]);  // move to window's lower left corner
    LG(w[2], w[1]);  // draw bottom
    LG(w[2], w[3]);  // draw right side
    LG(w[0], w[3]);  // draw top
    LG(w[0], w[1]);  // draw left side
}
// ==========================================================
//
// STYLE sets the line style in the graphics
// mode, in case a pattern other than a solid
// line is desired.  there are five choices.
//

void STYLE(uns type)
{
    gassert(ofs);              // make sure ofs defined
    switch (type){
        case 0:
            out(ofs, "\033`"); // solid
            break;
        case 1:
            out(ofs, "\033a"); // dots
            break;
        case 2:
            out(ofs, "\033b"); // dot-dashes
            break;
        case 3:
            out(ofs, "\033c"); // short dashes
            break;
        case 4:
            out(ofs, "\033d"); // long dashes
            break;
        default :
            err("unrecognized line style.  the five choices are\n"
                "     0: solid\n"
                "     1: dotted\n"
                "     2: dot-dashed\n"
                "     3: short dashes\n"
                "     4: long dashes");
            break;
    }
}
// ==========================================================
//
// the usual statement to put at the end of a
// graph-plotting program.  GMODE(off) will
// flush the buffer for the xterm
//

void END(void)
{
    if (xterm){
        GMODE(off);  // turns graph mode off; flushes buffer
    }
    hotkeys(off);    // turn hotkeys off if they're on
    AWIN();          // switches to alphanumeric window
}
// ==========================================================
