// basic 4th order runge-kutta, patterned after a
// numerical recipies piece, for nvar dependent
// variables in u and one independent variable t.
// supplied with u(t) and dudt(t) and a step size h,
// uses derivs(t, u, dudt) to find u(t+h).  note
// that derivs must be called by main to get initial
// values, allowing this routine to be used by an
// adaptive step-size routine (see rkqc).

#include "lorenz.h"

void rk(int nvar, dbl h, dbl t, vector<dbl>& u,
vector<dbl>& dudt, void derivs(dbl, vector<dbl>&, vector<dbl>&))
{
    dbl    hh;      // h/2 ; h is time step
    dbl    h6;      // h/6
    dbl    th;      // t + hh
    vector<dbl> ut (nvar);  // temp array for u
    vector<dbl> dut (nvar); // temp array for dudt
    vector<dbl> dum (nvar); // temp array for dudt

    hh = 0.5 * h; // half of h
    h6 = h / 6.0; // 1/6 of h
    th = t + hh;
    int i;
    for (i = 0 ; i < nvar ; i++){
        ut[i] = u[i] + hh * dudt[i]; // 1st step
    }
    derivs(th, ut, dut);             // 2nd step

    for (i = 0 ; i < nvar ; i++){
        ut[i] = u[i] + hh * dut[i];
    }
    derivs(th, ut, dum);             // 3rd step

    for (i = 0 ; i < nvar ; i++){
        ut[i] = u[i] + h * dum[i];
        dum[i] += dut[i];            // combine 2nd & 3rd
    }
    derivs(t+h, ut, dut);            // 4th step

    for (i = 0 ; i < nvar ; i++){    // add it all up
        u[i] += h6 * (dudt[i] + 2.0 * dum[i] + dut[i]);
    }
}
